@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2020 Dr. Jan-Philip Gehrcke. See LICENSE file for details.
+# Copyright 2012-2021 Dr. Jan-Philip Gehrcke. See LICENSE file for details.
 
 
 """
@@ -26,6 +26,7 @@ import codecs
 import logging
 import multiprocessing
 import multiprocessing.process
+import multiprocessing.reduction
 from itertools import chain
 
 try:
@@ -35,14 +36,16 @@ except ImportError:
 
 WINDOWS = sys.platform == "win32"
 
+FORK_MODE = multiprocessing.get_start_method()
+
 if WINDOWS:
-    import multiprocessing.reduction
     import msvcrt
 
 import gevent
 import gevent.os
 import gevent.lock
 import gevent.event
+import gevent.hub
 
 
 # Decide which method to use for transferring WinAPI pipe handles to children.
@@ -327,21 +330,39 @@ def _child(target, args, kwargs):
         # could even reset sigprocmask (Python 2.x does not have API for it, but
         # it could be done via ctypes).
         _reset_signal_handlers()
+
         # `gevent.reinit` calls `libev.ev_loop_fork()`, which reinitialises
         # the kernel state for backends that have one. Must be called in the
         # child before using further libev API.
         gevent.reinit()
+
         log.debug("Delete current hub's threadpool.")
         hub = gevent.get_hub()
         # Delete threadpool before hub destruction, otherwise `hub.destroy()`
         # might block forever upon `ThreadPool.kill()` as of gevent 1.0rc2.
         del hub.threadpool
         hub._threadpool = None
+
         # Destroy default event loop via `libev.ev_loop_destroy()` and delete
         # hub. This orphans all registered events and greenlets that have been
         # duplicated from the parent via fork().
         log.debug("Destroy hub and default loop.")
-        hub.destroy(destroy_loop=True)
+
+        # Ideally we would use `hub.destroy(destroy_loop=True)`, its internal
+        # sanity-checking is valuable. But seems to crash the child and I do
+        # not completely understand why -- are we really in a bad state or
+        # could this sanity-checking be more robust? There is no error thrown
+        # visible to the CPython interpreter -- the crash is more subtle; the
+        # child process just 'goes away'. I did not really debug this though
+        # (with for example strace):
+        # https://github.com/jgehrcke/gipc/issues/103#issuecomment-832885473
+        # Use `hub.loop.destroy()` instead. This is more brutal, pulls the
+        # libev event loop away _underneath_ the hub). To try to make this
+        # maybe a little less risky, use `gevent.hub.set_hub(None)` before loop
+        # destruction, so that gevent (hopefully) stops "using" the hub.
+        gevent.hub.set_hub(None)
+        hub.loop.destroy()
+
         # Create a new hub and a new default event loop via
         # `libev.gevent_ev_default_loop`.
         h = gevent.get_hub(default=True)
@@ -1040,6 +1061,37 @@ class _GIPCWriter(_GIPCHandle):
             self._write(struct.pack("!i", len(bindata)) + bindata)
 
 
+if not WINDOWS and FORK_MODE == 'spawn':
+    # When running in spawn mode on OSX multiprocessing uses
+    # spawnv_passfds to spawn the new process.
+    # Because of this, we need to explicitly pass our file
+    # descriptors. multiprocessing.reduction keeps track
+    # of what needs to be serialized by registering classes
+    # with functions to perform the serialization.
+    # See the register method in the python multiprocessing
+    # module for an example of how this is done.
+
+    def reduce_GIPCReader(reader):
+        df = multiprocessing.reduction.DupFd(reader._fd)
+        return (rebuild_GIPCReader, (df, reader._decoder))
+
+    def rebuild_GIPCReader(df, decoder):
+        fd = df.detach()
+        return _GIPCReader(fd, decoder)
+
+    multiprocessing.reduction.register(_GIPCReader, reduce_GIPCReader)
+
+    def reduce_GIPCWriter(writer):
+        df = multiprocessing.reduction.DupFd(writer._fd)
+        return (rebuild_GIPCWriter, (df, writer._encoder))
+
+    def rebuild_GIPCWriter(df, _encoder):
+        fd = df.detach()
+        return _GIPCWriter(fd, _encoder)
+
+    multiprocessing.reduction.register(_GIPCWriter, reduce_GIPCWriter)
+
+
 class _PairContext(tuple):
     """
     Generic context manager for a 2-tuple containing two entities supporting
